@@ -1,6 +1,9 @@
 #!/bin/sh
 
 TX=`which tx`
+PWD=`dirname "$0"`
+
+cd $PWD/..
 
 # In 'translator' mode files will contain empty translated texts
 # where translation is not available, we'll remove these later
@@ -8,9 +11,7 @@ TX=`which tx`
 # Note: there's a bug in txclib, so if the command below doesn't
 # work see https://github.com/transifex/transifex-client/commit/a80320735973dd608b48520bf3b89ad53e2b088b
 
-$TX --debug pull -a -f --mode translator
-
-PWD=`dirname "$0"`
+$TX pull -a -f --mode translator
 
 do_clean()
 {
@@ -32,15 +33,15 @@ do_clean()
 }
 
 # clean up translation files
-for file in $PWD/../program/localization/*/*.inc; do
+for file in program/localization/*/*.inc; do
     do_clean $file
 done
-for file in $PWD/../plugins/*/localization/*.inc; do
+for file in plugins/*/localization/*.inc; do
     do_clean $file
 done
 
 # remove empty localization files
-for file in $PWD/../program/localization/*/labels.inc; do grep -q -E '\$labels' $file || rm $file; done
-for file in $PWD/../program/localization/*/timezones.inc; do grep -q -E '\$labels' $file || rm $file; done
-for file in $PWD/../program/localization/*/messages.inc; do grep -q -E '\$messages' $file || rm $file; done
-for file in $PWD/../plugins/*/localization/*.inc; do grep -q -E '\$(labels|messages)' $file || rm $file; done
+for file in program/localization/*/labels.inc; do grep -q -E '\$labels' $file || rm $file; done
+for file in program/localization/*/timezones.inc; do grep -q -E '\$labels' $file || rm $file; done
+for file in program/localization/*/messages.inc; do grep -q -E '\$messages' $file || rm $file; done
+for file in plugins/*/localization/*.inc; do grep -q -E '\$(labels|messages)' $file || rm $file; done
